@@ -116,30 +116,40 @@ void segatex::segatex_write_to_my_policy()
 }
 void segatex::segatex_show_yum_update()
 {
-        status_label_1->setText("yum updating SELinx related rpms. Please wait for a while...");
+        status_label_1->setText("yum install/updating SELinx related rpms. Please wait for a while...");
 }
 
 void segatex::segatex_yum_update()
 {
 	using namespace std;
 	#define PWD_BUF 8600
-	//char buffer_d[PWD_BUF];
 	const char* yum_update_file="./yum_update";
-	//pwd_d=getcwd(buffer_d,sizeof(buffer_d));
 	ofstream os;
 	os.open(yum_update_file);
 	if(os){
+		FILE *read_fp_install;
 		FILE *read_fp;
 		#define SIZE_TMP 8192
 		char buffer[SIZE_TMP];
 		int chars_read;
 		memset(buffer, '\0', sizeof(buffer));
-		read_fp = popen("yum -y update selinux-policy* libse* policycoreutils checkpolicy", "r");
+		read_fp_install = popen("yum -y install selinux-policy* libsepol policycoreutils checkpolicy", "r");
+		if(read_fp_install != NULL){
+			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
+			while(chars_read>0){
+				buffer[chars_read -1] = '\0';
+				//os << "Reading:-" << endl;
+				os << buffer;
+				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp_install);
+			}
+			pclose(read_fp_install);
+		}
+		read_fp = popen("yum -y update selinux-policy* libsepol policycoreutils checkpolicy", "r");
 		if(read_fp != NULL){
 			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
 			while(chars_read>0){
 				buffer[chars_read -1] = '\0';
-				os << "Reading:-" << endl;
+				//os << "Reading:-" << endl;
 				os << buffer;
 				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
 			}
@@ -154,6 +164,73 @@ void segatex::segatex_yum_update()
 	}
 }
 
+
+void segatex::segatex_audit_2_allow()
+{
+	using namespace std;
+	#define PWD_BUF 8600
+	const char* audit_2_allow_file="./local.te";
+	ofstream os;
+	os.open(audit_2_allow_file);
+	if(os){
+		FILE *read_fp;
+		#define SIZE_TMP 8192
+		char buffer[SIZE_TMP];
+		int chars_read;
+		memset(buffer, '\0', sizeof(buffer));
+		read_fp = popen("audit2allow -m local -i /var/log/audit/audit.log", "r");
+		if(read_fp != NULL){
+			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			while(chars_read>0){
+				buffer[chars_read -1] = '\0';
+				//os << "Reading:-" << endl;
+				os << buffer;
+				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			}
+			pclose(read_fp);
+		}
+		os.close();
+		QFile file( audit_2_allow_file ); // Read the text from a file
+		if ( file.open( IO_ReadOnly ) ) {
+			QTextStream stream( &file );
+			textEdit1->setText( stream.read() );
+		}
+	}
+}
+
+void segatex::segatex_audit_2_why()
+{
+	using namespace std;
+	#define PWD_BUF 8600
+	const char* audit_2_why_file="./local_te_why";
+	ofstream os;
+	os.open(audit_2_why_file);
+	if(os){
+		FILE *read_fp;
+		#define SIZE_TMP 8192
+		char buffer[SIZE_TMP];
+		int chars_read;
+		memset(buffer, '\0', sizeof(buffer));
+		read_fp = popen("audit2why < /var/log/audit/audit.log", "r");
+		if(read_fp != NULL){
+			chars_read = fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			while(chars_read>0){
+				buffer[chars_read -1] = '\0';
+				//os << "Reading:-" << endl;
+				os << buffer;
+				chars_read= fread(buffer, sizeof(char), SIZE_TMP, read_fp);
+			}
+			pclose(read_fp);
+		}
+		os.close();
+		QFile file( audit_2_why_file ); // Read the text from a file
+		if ( file.open( IO_ReadOnly ) ) {
+			QTextStream stream( &file );
+			textEdit1->setText( stream.read() );
+		}
+	}
+}
+
 int segatex::segatex_set_time_real()
 {
         log_from_time = time((time_t*)0);
@@ -281,6 +358,8 @@ void segatex::segatex_show_interface()
 					const char* tclass = "tclass(.*)";
 					const char* tclass_2 = "[a-z0-9_]([^=]*)?";
 					const char* acc = "\\{(.*)\\}";
+					const char* role = ":[a-z0-9_]*_r";
+
 					regex denied_exp(den);
 					regex log_time_exp(log_time);
 					regex domain_exp(scontext);
@@ -290,6 +369,7 @@ void segatex::segatex_show_interface()
 					regex object_class_exp(tclass);
 					regex object_class_exp_2(tclass_2);
 					regex access_vector_exp(acc);
+					regex role_exp(role);
 
 					string log_time_string = "";
 					int log_time_int = 0;
@@ -305,6 +385,8 @@ void segatex::segatex_show_interface()
 					string object_class_pre = "";
 					string object_class = "";
 					string access_vector = "";
+					string role_str = "";
+					//string blank_str = "";
 					//multimap<string, string>macro_pre;
 					multimap<string, string> pattern_matched;
 					multimap<string, string>::iterator p_pattern_matched;
@@ -365,13 +447,26 @@ void segatex::segatex_show_interface()
 					vector<string> v_interface_raw;
 					vector<string>::iterator p_interface_raw, p_interface_raw_end;
 
+					vector<string> v_role;
+					vector<string>::iterator p_role, p_role_end;
+
 					for(p=v.begin(); p<v.end();p++){
 						string source_ready = *p;
+						//os_policy << source_ready << endl;
 						const char* source = source_ready.c_str();
 						cmatch what;
 						if(regex_search(source,what,denied_exp)){
 							if(regex_search(source,what,domain_exp)){
 								domain = what[0];
+								//os_policy << domain << endl;
+								const char* domain_char = domain.c_str();
+								if(regex_search(domain_char,what,role_exp)){
+									role_str = what[0];
+									string blank_str="";
+									role_str=role_str.replace(0,1,blank_str);
+									v_role.push_back(role_str.c_str());
+									//os_policy << role_str << endl;
+								}
 								int str_length = strlen(domain.c_str());
 								int i=str_length;
 								string str_rev="";
@@ -938,7 +1033,7 @@ void segatex::segatex_show_interface()
 					p_tmp_objclass_end=unique(v_tmp_objclass.begin(),v_tmp_objclass.end());
 
 					os_policy << "####################################################" << endl;
-					os_policy << "##### This file was generated by segatex2.07" << endl;
+					os_policy << "##### This file was generated by segatex2.08" << endl;
 					os_policy << "####################################################" << endl;
 					os_policy << "#### $make -f /usr/share/selinux/devel/Makefile" << endl;
 					os_policy << "#### will generate module in this directory." << endl;
@@ -983,7 +1078,6 @@ void segatex::segatex_show_interface()
 						}
 					}
 
-					os_policy << endl;
 					/*type ϡdomaintclassޤȤƽФ*/
 					/*κݡ󥿥ե˥ޥåΤȥޥåʤäΤޤȤƤ*/
 					int v_matched_tmp_domain_size=v_matched_tmp_domain.size();
@@ -998,7 +1092,14 @@ void segatex::segatex_show_interface()
 						string dom_str = *p_tmp_domain;
 						os_policy << "	type " << dom_str << ";" << endl;
 					}
-					os_policy << "}" << endl << endl;
+					//roleФ
+					sort(v_role.begin(),v_role.end());
+					p_role_end=unique(v_role.begin(),v_role.end());
+					for(p_role=v_role.begin(); p_role < p_role_end; p_role++){
+						string role_tmp_str= *p_role;
+						os_policy << "	role " << role_tmp_str << ";" << endl;
+					}
+					os_policy << "};" << endl;
 					/*ޥåʤäallowʸФ*/
 					cmatch what;
 					const char* m_snmfp = "(.*):([^[:space:]]*)([:space:])?";
